package sample;

import my.MyCity;
import my.MyCityNameMatcher;
import my.OrdinanceDesignatedCities;
import org.hamcrest.Matchers;
import org.junit.Test;

import java.util.List;
import java.util.stream.Collectors;

import static org.hamcrest.CoreMatchers.is;
import static org.hamcrest.Matchers.hasProperty;
import static org.hamcrest.Matchers.samePropertyValuesAs;
import static org.junit.Assert.assertThat;

/**
 * 主に「流れるようなアサーション」のHamcrestを使用したサンプルコードです。
 *
 * assertThatのstatic importが重複するため、このクラスにわけています。
 */
public class HamcrestTest {

    @Test
    public void コレクションの検証() throws Exception {
        List<MyCity> list = OrdinanceDesignatedCities.get().stream()
                .filter(city -> city.getPrefecture().equals("大阪府"))
                .collect(Collectors.toList());

        assertThat(list, Matchers.hasSize(2));

        assertThat(list.get(0).getCity(), Matchers.is("堺市"));
        assertThat(list.get(1).getCity(), Matchers.is("大阪市"));

        assertThat(list, Matchers.contains(list.get(0), list.get(1)));
        assertThat(list, Matchers.hasItems(list.get(0)));

        assertThat(list, Matchers.contains(
                Matchers.hasProperty("city", Matchers.is("堺市")),
                Matchers.hasProperty("city", Matchers.is("大阪市"))
        ));

        assertThat(list, Matchers.contains(
                MyCityNameMatcher.city("堺市"),
                MyCityNameMatcher.city("大阪市")
        ));
    }

    @Test
    public void HamcrestLibraryを使用したプロパティ検証() throws Exception {
        MyCity actual = new MyCity("大阪府", "大阪市");

        assertThat(actual, is(hasProperty("city", is("大阪市"))));
    }

    @Test
    public void HamcrestLibraryを使用したequalsを使用しない同値検証() throws Exception {
        MyCity actual = new MyCity("大阪府", "大阪市");
        MyCity expected = new MyCity("大阪府", "大阪市");

        assertThat(actual, is(samePropertyValuesAs(expected)));
    }
}
