package sample;

import my.MyCity;
import my.OrdinanceDesignatedCities;
import org.assertj.core.api.Assertions;
import org.assertj.core.groups.Tuple;
import org.junit.Test;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * 「流れるようなアサーション コレクションを検証しよう」のサンプルコードです。
 */
public class Section5CollectionTest {

    @Test
    public void AssertJでの検証() throws Exception {
        List<MyCity> list = OrdinanceDesignatedCities.get().stream()
                .filter(city -> city.getPrefecture().equals("大阪府"))
                .collect(Collectors.toList());

        Assertions.assertThat(list)
                .extracting(MyCity::getCity)
                .containsExactly("堺市", "大阪市");

        Assertions.assertThat(list)
                .extracting(MyCity::getPrefecture, MyCity::getCity)
                .containsExactly(
                        Tuple.tuple("大阪府", "堺市"),
                        Tuple.tuple("大阪府", "大阪市"));

        Assertions.assertThat(list)
                .flatExtracting(MyCity::getPrefecture, MyCity::getCity)
                .containsExactly(
                        "大阪府", "堺市",
                        "大阪府", "大阪市");
    }

    @Test
    public void コレクション_フィールドを扱う検証3() throws Exception {
        List<MyCity> actual = OrdinanceDesignatedCities.get();
        MyCity expected = new MyCity("日本", "大阪府", "大阪市");

        assertThat(actual)
                .usingFieldByFieldElementComparator()
                .contains(expected);
    }

    @Test
    public void filteredOn() throws Exception {
        List<MyCity> list = OrdinanceDesignatedCities.get();

        Assertions.assertThat(list)
                .hasSize(20)
                .filteredOn(city ->
                        Objects.equals(city.getPrefecture(), "大阪府"))
                .hasSize(2);
    }

}
