package sample;

import org.assertj.core.api.Assertions;
import org.junit.Assert;
import org.junit.Rule;
import org.junit.Test;
import org.junit.rules.ExpectedException;

/**
 * 「流れるようなアサーション 例外を検証しよう」のサンプルコードです。
 */
public class Section5ExceptionTest {

    /**
     * JUnit 3時代の例外検証
     */
    @Test
    public void 例外を検証するお決まりのコーディング() throws Exception {
        try {
            throwingExceptionProcess();
            // 例外が起こらない場合はテストを失敗させる
            Assert.fail();
        } catch (Exception e) {
            // 例外に対する検証
            Assert.assertEquals(IllegalStateException.class, e.getClass());
        }
    }

    /**
     * JUnit 4のTestアノテーションでの例外検証
     */
    @Test(expected = IllegalStateException.class)
    public void アノテーションによる検証() throws Exception {
        throwingExceptionProcess();
    }

    @Rule
    public ExpectedException thrown = ExpectedException.none();

    /**
     * JUnit 4のExpectedExceptionルールを使用した例外検証
     */
    @Test
    public void Ruleによる検証() throws Exception {
        thrown.expect(IllegalStateException.class);
        throwingExceptionProcess();
    }

    /**
     * AssertJの例外検証
     */
    @Test
    public void AssertJの検証_assertThatThrownBy() throws Exception {
        Assertions.assertThatThrownBy(() ->
                throwingExceptionProcess()
        ).isInstanceOf(IllegalStateException.class);
    }

    /**
     * AssertJの例外検証（誌面では紹介していないものです）
     */
    @Test
    public void AssertJの検証_assertThatExceptionOfType() throws Exception {
        Assertions.assertThatExceptionOfType(IllegalStateException.class)
                .isThrownBy(() -> throwingExceptionProcess());
    }

    /**
     * 例外を投げるメソッド
     */
    private void throwingExceptionProcess() {
        throw new IllegalStateException("実行時例外");
    }
}
