package sample;

import my.MyCity;
import org.junit.Test;

import java.math.BigDecimal;
import java.util.Comparator;

import static org.assertj.core.api.Assertions.assertThat;

/**
 * 「流れるようなアサーション オブジェクトを比較しよう」のサンプルコードです。
 */
public class Section5ObjectCompareTest {

    @Test
    public void equalsが実装されている場合() throws Exception {
        BigDecimal actual = new BigDecimal("12.345");
        BigDecimal expected = new BigDecimal("12.345");

        assertThat(actual).isEqualTo(expected);
    }

    @Test
    public void すべてのフィールドで比較検証() throws Exception {
        MyCity actual = new MyCity("大阪府", "大阪市");
        MyCity expected = new MyCity("大阪府", "大阪市");

        assertThat(actual).isEqualToComparingFieldByField(expected);
    }

    @Test
    public void 特定のフィールドのみで比較検証() throws Exception {
        MyCity actual = new MyCity("日本", "大阪府", "大阪市");
        MyCity expected = new MyCity("???", "大阪府", "大阪市");

        assertThat(actual)
                .isEqualToComparingOnlyGivenFields(expected,
                        "prefecture", "city")
                .isEqualToIgnoringGivenFields(expected, "country");

        assertThat(actual)
                .isEqualToIgnoringNullFields(new MyCity(null, "大阪府", null));
    }


    @Test
    public void Comparatorを使用した比較検証() throws Exception {
        MyCity actual = new MyCity("大阪府", "大阪市");
        MyCity expected = new MyCity("大阪府", "堺市");

        Comparator<MyCity> 都道府県のみで比較 = (o1, o2) ->
                o1.getPrefecture().compareTo(o2.getPrefecture());

        assertThat(actual)
                .usingComparator(都道府県のみで比較)
                .isEqualTo(expected);
    }
}
